/*
 * 05/21/2003 - 13:55:17
 *
 * GlobalActionCollection.java -
 * Copyright (C) 2003 Dreux Loic
 * dreuxl@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.analyse.core.gui.action;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.Hashtable;
import java.util.Observable;
import java.util.Observer;

import javax.swing.Action;
import javax.swing.KeyStroke;

import org.analyse.core.modules.AnalysePanel;
import org.analyse.core.util.GUIUtilities;
import org.analyse.main.Main;

/**
 * Cette classe regroupe toutes les actions globales.
 */
public class GlobalActionCollection implements Observer
{
    private Hashtable actionCollection;

    private MainActionListener mainActionListener;

    /**
     * Crer les diffrentes actions.
     */
    public GlobalActionCollection()
    {
        mainActionListener = new MainActionListener();
        actionCollection = new Hashtable();

        BasicAction action;

        action = new BasicAction("Nouveau", "Crer un nouveau document", "NEW",
                GUIUtilities.getImageIcon("New.png"), 'n', KeyStroke
                        .getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction("Ouvrir", "Ouvrir un document existant",
                "OPEN", GUIUtilities.getImageIcon("Open.png"), 'o', KeyStroke
                        .getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction("Sauvegarder", "Sauvegarder le document",
                "SAVE", GUIUtilities.getImageIcon("Save.png"), 's', KeyStroke
                        .getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction("Sauvegarder sous...",
                "Sauvegarder le document dans un nouveau fichier", "SAVEAS",
                GUIUtilities.getImageIcon("Saveas.png"), 'a', KeyStroke
                        .getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction("Quitter", "Quitter l'application", "QUIT",
                null, 'q', KeyStroke.getKeyStroke(KeyEvent.VK_Q,
                        ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction("Annuler", "Annuler la dernire opration",
                "UNDO", GUIUtilities.getImageIcon("Undo.png"), 'a', KeyStroke
                        .getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction("Rtablir", "Rtablir la dernire opration",
                "REDO", GUIUtilities.getImageIcon("Redo.png"), 'r', KeyStroke
                        .getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction(
                "Couper",
                "Supprime le ou les lment(s) slectionn(s) et les copie vers le press-papier",
                "CUT", GUIUtilities.getImageIcon("Cut.png"), 'c', KeyStroke
                        .getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction(
                "Coller",
                "Insert le ou les lment(s) que vous avez copi ou coup  l'emplacement slectionn",
                "PASTE", GUIUtilities.getImageIcon("Paste.png"), 'l', KeyStroke
                        .getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction(
                "Copier",
                "Copie le ou les lment(s) slectionn(s) vers le press-papier",
                "COPY", GUIUtilities.getImageIcon("Copy.png"), 'p', KeyStroke
                        .getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction("Afficher le Navigateur", "Afficher le Navigateur", "SHOWHIDE_NAVIGATOR",
                GUIUtilities.getImageIcon("Tree.png"), 0, null);
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);

        action = new BasicAction("A Propos", "A Propos", "ABOUT", GUIUtilities
                .getImageIcon("About.png"), 0, null);
        action.addActionListener(mainActionListener);
        actionCollection
                .put(action.getValue(Action.ACTION_COMMAND_KEY), action);
    }

    /**
     * Retourne une BasicAction utilisable dans un composant graphique.
     * 
     * @param ID
     *            identifiant de l'action
     */
    public BasicAction getAction(String ID)
    {
        return (BasicAction) (actionCollection.get(ID));
    }

    /**
     * Met  jour l'intercae : active ou dsactive les boutons Undo, Redo, Cut,
     * Copy et Paste.
     */
    public void update(Observable o, Object arg)
    {
        AnalysePanel p = Main.analyseFrame.getCurrentPanel();

        getAction("REDO").setEnabled(p.getRedoEnabled());
        getAction("UNDO").setEnabled(p.getUndoEnabled());

        getAction("CUT").setEnabled(p.getCopyEnabled() && p.getPasteEnabled());
        getAction("COPY").setEnabled(p.getCopyEnabled());
        getAction("PASTE").setEnabled(p.getPasteEnabled());
    }
}