/*
 * 05/02/2002 - 20:15:18
 *
 * ZGraphique -
 * Copyright (C) 2002 Dreux Loic
 * dreuxl@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.analyse.core.gui.zgraph;

import java.awt.AWTEvent;
import java.awt.AWTEventMulticaster;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.JComponent;

/**
 * ZGraphique est un composant permettant d'afficher des lments graphiques, de
 * les dplacer, de les lier. <br>
 * <br>
 * Il est conseiller d'utiliser ce composant dans un <code>JScrollPane<code>.
 * 
 *<pre>
 * Container c = getContentPane();
 * c.setLayout(new BorderLayout());
 * c.add(BorderLayout.CENTER, new JScrollPane(new ZGraphique()));
 * </pre>
 */
public class ZGraphique extends JComponent implements MouseListener,
        MouseMotionListener, Observer
{
    /** Vecteur de <code>ZElement</code> */
    private Vector zelements = new Vector(20, 5);

    /** Vecteur de <code>ZLien</code> */
    private Vector zliens = new Vector(20, 5);

    /**
     * Cette variable sert pour le glisser-dposer. Lorsque l'on clic sur un
     * <code>ZElement</code> la variable est lie  ce <code>ZElement</code>
     * jusqu' ce qu'on relache la souris
     */
    private ZElement elementPress = null;

    /**
     * Cette variable est lie vers un <code>ZElement</code> sur lequel on a
     * cliqu jusque l'on clique dans le vide ou sur un autre lment.
     */
    protected ZElement elementClic = null;

    /**
     * Cette variable est lie vers un <code>ZLien</code> sur lequel on a
     * cliqu jusque l'on clique dans le vide ou sur un autre lment.
     */
    protected ZLien lienClic = null;

    /**
     * Cette variable est utilise lors de la cration d'un lien.
     */
    private ZLien lienTemp;

    /** Utilis pour le dplacement des lments */
    private int xCentrage, yCentrage;

    /** Possibilit d'intragir sur le composant grace  la souris. */
    private boolean enabled = true;

    /** Enclenche le mode 'cration de lien' */
    private boolean creationLien = false;

    /** Indique si en mode 'cration de lien', l'utilisateur a cliqu */
    private boolean creationLienClic = false;

    /** Gestion des vnement */
    private ActionListener actionListener;

    /** Pour la position du lien */
    private int x1, y1, x2, y2;

    /**
     * Dfinie un <code>ZGraphique</code> avec les paramtres par dfaut.
     */
    public ZGraphique()
    {
        addMouseListener(this);
        setSize(getPreferredSize());
        setBackground(Color.white);
    }

    /**
     * Active ou dsactive l'intraction de l'utilisateur avec le graphique.
     */
    public void setEnabled(boolean enabled)
    {
        this.enabled = enabled;
    }

    /**
     * Retourne la variable enabled.
     */
    public boolean getEnabled()
    {
        return enabled;
    }

    /**
     * Retourne le nombre de <code>ZElement</code>.
     */
    public int sizeElements()
    {
        return zelements.size();
    }

    /**
     * Retourne le nombre de liens.
     */
    public int sizeLien()
    {
        return zliens.size();
    }

    /**
     * Retourne une Enumeration des <code>ZElement</code>.
     */
    public Enumeration enumElements()
    {
        return zelements.elements();
    }

    /**
     * Retourne une numration des <code>ZLien</code>
     */
    public Enumeration enumLiens()
    {
        return zliens.elements();
    }

    /**
     * Ajoute un <code>ZElement</code> dans le composant.
     */
    public void addElement(ZElement element)
    {
        zelements.add(element);
        element.majObserver(this);
        repaint();
    }

    /**
     * Ajoute plusiers <code>ZElement</code> dans le composant.
     */
    public void addElements(Vector elements)
    {
        Enumeration e = elements.elements();
        while (e.hasMoreElements())
            addElement((ZElement) e.nextElement());
    }

    /**
     * Supprime un <code>ZElement</code> du composant.
     */
    public void removeElement(ZElement element)
    {
        enleverFocus();

        Enumeration e = zliens.elements();
        Vector supp = new Vector();
        while (e.hasMoreElements()) {
            ZLien l = (ZLien) (e.nextElement());
            if (l.getElement(ZLien.ELEM1).equals(element)
                    || l.getElement(ZLien.ELEM2).equals(element))
                supp.add(l);
        }
        removeLiens(supp);
        zelements.remove(element);
        repaint();
    }

    /**
     * Ajoute un <code>ZLien</code> dans le composant.
     */
    public void addLien(ZLien lien)
    {
        if (lien.getElement(ZLien.ELEM1) != null
                && lien.getElement(ZLien.ELEM2) != null
                && lien.getElement(ZLien.ELEM1) != lien.getElement(ZLien.ELEM2)
                && !zliens.contains(lien))
            zliens.add(lien);
        repaint();
    }

    /**
     * Ajoute plusiers <code>ZLien</code> dans le composant.
     */
    public void addLiens(Vector liens)
    {
        Enumeration e = liens.elements();
        while (e.hasMoreElements())
            addLien((ZLien) e.nextElement());
    }

    /**
     * Supprime un <code>ZLien</code> du composant.
     */
    public void removeLien(ZLien lien)
    {
        enleverFocus();
        zliens.remove(lien);
        repaint();
    }

    /**
     * Supprime plusieurs <code>ZLien</code> du composant.
     */
    public void removeLiens(Vector liens)
    {
        for (Enumeration e = liens.elements(); e.hasMoreElements();)
            removeLien((ZLien) e.nextElement());
    }

    /**
     * Retourne un <code>ZLien</code> du composant.
     * 
     * @param i
     *            index du lien  retourner
     */
    public ZLien getLien(int i)
    {
        return ((ZLien) (zliens.elementAt(i)));
    }

    /**
     * Retourne le lien qui a le focus.
     */
    public ZLien getLienFocus()
    {
        return lienClic;
    }

    /**
     * Retourne une Enumeration des <code>ZLien</cdoe>.
     */
    public Enumeration elementsZLiens()
    {
        return zliens.elements();
    }

    /**
     * Retourne un <code>ZElement</code> du composant.
     * 
     * @param i
     *            index du composant  retourner
     */
    public ZElement getElement(int i)
    {
        return ((ZElement) (zelements.elementAt(i)));
    }

    /**
     * Retourne le composant qui a le focus.
     */
    public ZElement getElementFocus()
    {
        return elementClic;
    }

    /**
     * Retourne une Enumeration des <code>ZElement</code>.
     */
    public Enumeration elementsZElements()
    {
        return zelements.elements();
    }

    /**
     * Retourne le premier composant se trouvant  la position x, y.
     */
    public Object getObjectFromLocation(int x, int y)
    {
        for (int i = zelements.size() - 1; i >= 0; i--)
            if (x > getElement(i).getX()
                    && x < getElement(i).getWidth() + getElement(i).getX()
                    && y > getElement(i).getY()
                    && y < getElement(i).getHeight() + getElement(i).getY())
                return getElement(i);

        for (int i = 0; i < zliens.size(); i++)
            if (getLien(i).isSelected(x, y))
                return getLien(i);
        return null;
    }

    /**
     * Enlve le focus de tous les elements.
     */
    public void enleverFocus()
    {
        elementClic = null;
        lienClic = null;
    }

    /**
     * Enclenche le mode de "cration de liens". Il est alors possible de lie
     * deux elements dans le graphe.
     */
    public void creerLien(ZLien lien)
    {
        enleverFocus();
        lienTemp = lien;
        lienTemp.setElement(null, ZLien.ELEM1);
        lienTemp.setElement(null, ZLien.ELEM2);
        if (zliens.contains(lien))
            removeLien(lien);
        creationLien = true;
        creationLienClic = false;
    }

    /**
     * Dsenclence le mode "cration de liens".
     */
    public void annulerCreerLien()
    {
        enleverFocus();

        lienTemp = null;
        creationLien = false;
        creationLienClic = false;
    }

    /**
     * Cette mthode indique si l'on peut crer un lien entre deux lments.
     */
    public boolean peutCreerLien(ZElement elem1, ZElement elem2)
    {
        ZLien lien;

        Enumeration e = zliens.elements();
        while (e.hasMoreElements()) {
            lien = (ZLien) e.nextElement();
            if (lien.getElement(ZLien.ELEM1) == elem1
                    && lien.getElement(ZLien.ELEM2) == elem2) {
                return false;
            }
        }
        return true;
    }

    public String infoZLiens()
    {
        String s = "{";
        for (Enumeration e = elementsZLiens(); e.hasMoreElements();) {
            ZLien lien = (ZLien) e.nextElement();
            s += "\n" + lien;
        }
        return s + "\n}";
    }

    public void clearAll()
    {
        enleverFocus();
        zelements = new Vector(10, 5);
        zliens = new Vector(10, 5);
        repaint();
    }

    /**
     * Dessine le composant.
     */
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);

        g.setColor(getBackground());
        g.fillRect(0, 0, getWidth(), getHeight());

        Enumeration e;

        // TODO inverser
        e = zelements.elements();
        while (e.hasMoreElements())
            ((ZElement) (e.nextElement())).paint(g);
        
        e = zliens.elements();
        while (e.hasMoreElements())
            ((ZLien) (e.nextElement())).paint(g);

        if (elementClic != null)
            elementClic.paintFocus(g);
        if (lienClic != null)
            lienClic.paintFocus(g);

        if (creationLien && creationLienClic) {
            Graphics2D g2d = (Graphics2D) g;
            Rectangle2D r;
            g2d.setColor(Color.black);

            Line2D l = new Line2D.Double(x1, y1, x2, y2);
            g2d.draw(l);

            r = new Rectangle2D.Double(x1 - 1, y1 - 1, 4, 4);
            g2d.fill(r);

            r = new Rectangle2D.Double(x2 - 2, y2 - 1, 4, 4);
            g2d.fill(r);
        }
    }

    public Dimension getPreferredSize()
    {
        int x = 0, y = 0;
        for (int i = 0; i < zelements.size(); i++) {
            x = getElement(i).getX() + getElement(i).getWidth() > x ? getElement(
                    i).getX()
                    + getElement(i).getWidth()
                    : x;
            y = getElement(i).getY() + getElement(i).getHeight() > y ? getElement(
                    i).getY()
                    + getElement(i).getHeight()
                    : y;
        }
        return new Dimension(x + 30, y + 30);
    }

    public void update(Observable o, Object arg)
    {
        this.repaint();
    }

    public void addActionListener(ActionListener listener)
    {
        actionListener = AWTEventMulticaster.add(actionListener, listener);
        enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    }

    public void removeActionListener(ActionListener listener)
    {
        actionListener = AWTEventMulticaster.remove(actionListener, listener);
    }

    public void mouseClicked(MouseEvent e)
    {
    }

    public void mousePressed(MouseEvent e)
    {
        addMouseMotionListener(this);

        if (creationLien && enabled) {
            creationLienClic = true;
            int x = e.getX();
            int y = e.getY();

            x1 = e.getX();
            y1 = e.getY();

            x2 = e.getX();
            y2 = e.getY();

            for (int i = zelements.size() - 1; i >= 0; i--) {
                if (e.getX() > getElement(i).getX()
                        && e.getX() < getElement(i).getWidth()
                                + getElement(i).getX()
                        && e.getY() > getElement(i).getY()
                        && e.getY() < getElement(i).getHeight()
                                + getElement(i).getY()) {
                    lienTemp.setElement(getElement(i), ZLien.ELEM1);
                    break;
                }
            }

            repaint();

        } else if (enabled) {
            elementPress = null;
            elementClic = null;
            lienClic = null;

            int x = e.getX();
            int y = e.getY();

            for (int i = zelements.size() - 1; i >= 0; i--) {
                if (e.getX() > getElement(i).getX()
                        && e.getX() < getElement(i).getWidth()
                                + getElement(i).getX()
                        && e.getY() > getElement(i).getY()
                        && e.getY() < getElement(i).getHeight()
                                + getElement(i).getY()) {
                    elementPress = getElement(i);
                    elementClic = getElement(i);
                    xCentrage = x - elementPress.getX();
                    yCentrage = y - elementPress.getY();
                    break;
                }
            }
            if (elementClic == null) {
                for (int i = 0; i < zliens.size(); i++) {
                    if (getLien(i).isSelected(x, y))
                        lienClic = getLien(i);
                }
            }
        }
    }

    public void mouseReleased(MouseEvent e)
    {
        removeMouseMotionListener(this);

        if (creationLien && enabled) {
            creationLien = false;
            for (int i = zelements.size() - 1; i >= 0; i--) {
                if (e.getX() > getElement(i).getX()
                        && e.getX() < getElement(i).getWidth()
                                + getElement(i).getX()
                        && e.getY() > getElement(i).getY()
                        && e.getY() < getElement(i).getHeight()
                                + getElement(i).getY()) {
                    lienTemp.setElement(getElement(i), ZLien.ELEM2);
                    x2 = e.getX();
                    y2 = e.getY();
                    break;
                }
            }
            if (peutCreerLien(lienTemp.getElement(ZLien.ELEM1), lienTemp
                    .getElement(ZLien.ELEM2)))
                addLien(lienTemp);
            else
                lienTemp.clearElement();
            repaint();
        } else if (enabled) {
            elementPress = null;
            repaint();
        }
        if (actionListener != null)
            actionListener.actionPerformed(new ActionEvent(this,
                    ActionEvent.ACTION_PERFORMED, ""));

        /* Correction Bug Graphique */
        if (getPreferredSize().height > getSize().height
                || getPreferredSize().width > getSize().width
        /*
         * || getPreferredSize().height < getSize().height ||
         * getPreferredSize().width < getSize().width
         */)
            this.setSize(getPreferredSize());

    }

    public void mouseDragged(MouseEvent e)
    {
        Point point = new Point();
        
        if (creationLien && enabled) {
            x2 = e.getX();
            y2 = e.getY();
            repaint();
        } else if (enabled) {
            if (elementPress != null) {
                if (e.getX() - xCentrage < 0)
                    point.x = 0;
                else
                    point.x = e.getX() - xCentrage;

                if (e.getY() - yCentrage < 0)
                    point.y = 0;
                else
                    point.y = e.getY() - yCentrage;
                
                elementPress.setPosition(point);
                repaint();
            }
        }
    }

    public void mouseEntered(MouseEvent e)
    {
    }

    public void mouseExited(MouseEvent e)
    {
    }

    public void mouseMoved(MouseEvent e)
    {
    }
}