/*
 * 05/05/2002 - 22:26:20
 * 
 * ZLien - Copyright (C) 2002 Dreux Loic dreuxl@free.fr
 * 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.analyse.core.gui.zgraph;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

public abstract class ZLien
{
    public static final int ELEM1 = 0;

    public static final int ELEM2 = 1;

    /** premier <code>ZElement</code>  lier */
    protected ZElement elem1;

    /** deuxime <code>ZElement</code>  lier */
    protected ZElement elem2;

    /** Position x1 */
    protected int x1;

    /** Position x2 */
    protected int x2;

    /** Position y1 */
    protected int y1;

    /** Position y2 */
    protected int y2;

    /**
     * Dfinie un <code>ZLien</code> avec les deux <code>ZElement</code>
     * qu'il lie
     * 
     * @param elem1
     *            premier <code>ZElement</code>  lier
     * @param elem2
     *            deuxime <code>ZElement</code>  lier
     */
    public ZLien(ZElement elem1, ZElement elem2)
    {
        this.elem1 = elem1;
        this.elem2 = elem2;
    }

    /**
     * Dfinie un <code> ZLien </code> sans argument. Ce lien est destin  tre
     * li par la suite dans le <code> ZGraphique </code>
     */
    public ZLien()
    {
        this(null, null);
    }

    /** Change l'lment 1 */
    public void setElement(ZElement elem, int index)
    {
        if (index == 0)
            this.elem1 = elem;
        else if (index == 1)
            this.elem2 = elem;
        else
            throw new IllegalArgumentException("");
    }

    /** Retourne l'lment 1 */
    public ZElement getElement(int index)
    {
        if (index == 0)
            return elem1;
        else if (index == 1)
            return elem2;

        throw new IllegalArgumentException("");
    }

    /**
     * Cette mthode est appele avant la suppression du lien.
     */
    public abstract void clearElement();

    /**
     * Redessine le composant dans un <code>ZGraphique</code> Attention : il
     * est important d'appeler une mthode de calcul des coordonnes des points :
     * soit calculPositionsDefaut() ou calculPositionsCentre().
     * 
     * @param g
     *            Graphics du <code>ZGraphique</code>
     */
    public abstract void paint(Graphics g);

    /**
     * <br>
     * Inverse les deux Elements : elem1 devient elem2 elem2 devient elem1</br>
     */
    public void inverseZElements()
    {
        ZElement elem = elem1;
        elem1 = elem2;
        elem2 = elem;
    }

    /**
     * Dessine le focus du composant s'il est slctionn
     */
    public void paintFocus(Graphics g)
    {
        Graphics2D g2d = (Graphics2D) g;
        Rectangle2D r;

        r = new Rectangle2D.Double(x1 - 1, y1 - 1, 4, 4);
        g2d.setColor(Color.black);
        g2d.fill(r);

        r = new Rectangle2D.Double(x2 - 2, y2 - 1, 4, 4);
        g2d.setColor(Color.black);
        g2d.fill(r);
    }

    /** Calcul des coordonnes des deux extrmits par dfaut */
    protected void updateDefaultLocation()
    {
        //Rcupration des centres
        int centrex1 = (elem1.getX() + elem1.getWidth() / 2);
        int centrey1 = (elem1.getY() + elem1.getHeight() / 2);
        int centrex2 = (elem2.getX() + elem2.getWidth() / 2);
        int centrey2 = (elem2.getY() + elem2.getHeight() / 2);

        int hauteur = centrex2 - centrex1;
        int largeur = centrey2 - centrey1;

        if (Math.abs(hauteur) > Math.abs(largeur)) {
            x1 = hauteur > 0 ? elem1.getX() + elem1.getWidth() : elem1.getX();
            y1 = elem1.getY() + elem1.getHeight() / 2;
            x2 = hauteur < 0 ? elem2.getX() + elem2.getWidth() : elem2.getX();
            y2 = elem2.getY() + elem2.getHeight() / 2;
        } else {
            x1 = elem1.getX() + elem1.getWidth() / 2;
            y1 = largeur > 0 ? elem1.getY() + elem1.getHeight() : elem1.getY();
            x2 = elem2.getX() + elem2.getWidth() / 2;
            y2 = largeur < 0 ? elem2.getY() + elem2.getHeight() : elem2.getY();
        }
    }

    /** Calcul des coordonnes par rapport au centre */
    public void updateLocation()
    {
        double theta, l1, l2, l3, diagonale;

        double thetas[] = new double[4];

        int pointx, pointy;

        //Rcupration des centres
        int centrex1 = (elem1.getX() + elem1.getWidth() / 2);
        int centrey1 = (elem1.getY() + elem1.getHeight() / 2);
        int centrex2 = (elem2.getX() + elem2.getWidth() / 2);
        int centrey2 = (elem2.getY() + elem2.getHeight() / 2);

        // premier rectangle

        diagonale = Math.sqrt((elem1.getWidth()) * (elem1.getWidth())
                + (elem1.getHeight()) * (elem1.getHeight()));

        thetas[0] = Math.acos(elem1.getWidth() / diagonale);
        thetas[1] = Math.acos(-elem1.getWidth() / diagonale);
        thetas[2] = 2 * Math.PI - thetas[1];
        thetas[3] = 2 * Math.PI - thetas[0];

        l1 = centrex2 - centrex1;
        l2 = centrey2 - centrey1;
        l3 = Math.sqrt((centrex1 - centrex2) * (centrex1 - centrex2)
                + (centrey1 - centrey2) * (centrey1 - centrey2));

        theta = Math.acos((double) (l1 / l3));

        if (centrey1 > centrey2)
            theta = 2 * Math.PI - theta;

        if (theta < thetas[0] || theta >= thetas[3]) {
            y1 = (int) (elem1.getY() + elem1.getHeight() / 2 + (((elem1
                    .getWidth() / 2) * l2) / l1));
            x1 = elem1.getX() + elem1.getWidth();
        } else if (theta >= thetas[1] && theta < thetas[2]) {
            y1 = (int) (elem1.getY() + elem1.getHeight() / 2 - (((elem1
                    .getWidth() / 2) * l2) / l1));
            x1 = elem1.getX();
        } else if (theta >= thetas[0] && theta < thetas[1]) {
            y1 = elem1.getY() + elem1.getHeight();
            x1 = (int) (elem1.getX() + elem1.getWidth() / 2 + (((elem1
                    .getHeight() / 2) * l1) / l2));
        } else {
            y1 = elem1.getY();
            x1 = (int) (elem1.getX() + elem1.getWidth() / 2 - (((elem1
                    .getHeight() / 2) * l1) / l2));
        }

        // deuxime rectangle

        theta = 2 * Math.PI - theta;

        diagonale = Math.sqrt((elem2.getWidth()) * (elem2.getWidth())
                + (elem2.getHeight()) * (elem2.getHeight()));

        thetas[0] = Math.acos(elem2.getWidth() / diagonale);
        thetas[1] = Math.acos(-elem2.getWidth() / diagonale);
        thetas[2] = 2 * Math.PI - thetas[1];
        thetas[3] = 2 * Math.PI - thetas[0];

        if (theta < thetas[0] || theta >= thetas[3]) {
            y2 = (int) (elem2.getY() + elem2.getHeight() / 2 - (((elem2
                    .getWidth() / 2) * l2) / l1));
            x2 = elem2.getX();
        } else if (theta >= thetas[1] && theta < thetas[2]) {
            y2 = (int) (elem2.getY() + elem2.getHeight() / 2 + (((elem2
                    .getWidth() / 2) * l2) / l1));
            x2 = elem2.getX() + elem2.getWidth();
        } else if (theta >= thetas[0] && theta < thetas[1]) {
            y2 = elem2.getY() + elem2.getHeight();
            x2 = (int) (elem2.getX() + elem2.getWidth() / 2 + (((elem2
                    .getHeight() / 2) * l1) / l2));
        } else {
            y2 = elem2.getY();
            x2 = (int) (elem2.getX() + elem2.getWidth() / 2 - (((elem2
                    .getHeight() / 2) * l1) / l2));
        }
    }

    public int getX1()
    {
        return x1;
    }

    public int getY1()
    {
        return y1;
    }

    public int getX2()
    {
        return x2;
    }

    public int getY2()
    {
        return y2;
    }

    public boolean isSelected(int mousex, int mousey)
    {
        double a, b, c, h, g;

        a = Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
        b = Math.sqrt((x1 - mousex) * (x1 - mousex) + (y1 - mousey)
                * (y1 - mousey));
        c = Math.sqrt((mousex - x2) * (mousex - x2) + (mousey - y2)
                * (mousey - y2));

        h = a / 2 - (b * b) / (2 * a) + c * c / (2 * a);

        g = Math.sqrt(c * c - h * h);
        if (g < 10
                && (mousex > x1 - 10 && mousex < x2 + 10 || mousex < x1 + 10
                        && mousex > x2 - 10)
                && (mousey > y1 - 10 && mousey < y2 + 10 || mousey < y1 + 10
                        && mousey > y2 - 10))
            return true;
        return false;
    }

    public void notifyZLien()
    {
        elem1.changementLien();
        elem1.notifyZElement();
    }

    public String toString()
    {
        return "ZLien, " + info();
    }

    public String info()
    {
        return "Element 1 : {" + elem1 + "}, Element 2 : {" + elem2
                + "}, Point1(" + x1 + ";" + y1 + "), Point2(" + x2 + ";" + y2
                + ")";
    }
}