/*
 * 05/16/2003 - 10:29:03
 * 
 * GUIUtilities.java - Copyright (C) 2003 Dreux Loic dreuxl@free.fr
 * 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.analyse.core.util;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.Toolkit;
import java.io.InputStream;
import java.net.URL;
import java.util.Hashtable;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;

import org.analyse.core.gui.AnalyseFrame;
import org.analyse.core.gui.HtmlWindow;

public class GUIUtilities
{
	// Hashtable contenant les images dj charge
	// Permet de ne pas recharger les images  chaque
	// fois dans un soucis de performance.
    private static Hashtable hashtable = new Hashtable();
	// Hashtable contenant les fonts dj charge
	private static Hashtable fonts = new Hashtable();

    /**
     * Centre un composant au mileu de l'cran
     * 
     * @param compo
     *            composant  centrer
     */
    public static void centerComponent(Component compo)
    {
        compo.setLocation(new Point((getScreenDimension().width - compo
                .getSize().width) / 2, (getScreenDimension().height - compo
                .getSize().height) / 2));
    }

    /**
     * Dtermine la taille de l'cran grace au toolkit
     * 
     * @return rsolution de l'cran
     */
    public static Dimension getScreenDimension()
    {
        return Toolkit.getDefaultToolkit().getScreenSize();
    }

    public static void error(String text)
    {
        JOptionPane.showMessageDialog(org.analyse.main.Main.analyseFrame, text,
                "Analyse", JOptionPane.ERROR_MESSAGE);
    }

    public static void message(String text)
    {
        JOptionPane.showMessageDialog(org.analyse.main.Main.analyseFrame, text,
                "Analyse", JOptionPane.INFORMATION_MESSAGE);
    }
    
	public static boolean question(String question) {
		return JOptionPane.showConfirmDialog(org.analyse.main.Main.analyseFrame, question, "Analyse", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
	}

    public static void messageHTML(String text)
    {
    	HtmlWindow f = new HtmlWindow(text, 500, 400,
                true, false);
    }
    
    public static void messageHTML(String text, boolean closeAfter2sec)
    {
    	HtmlWindow f = new HtmlWindow(text, 500, 400,
                true, closeAfter2sec);
    }
    
    public static void messageHTML(String text, boolean closeAfter2sec, int width, int height)
    {
    	HtmlWindow f = new HtmlWindow(text, width, height,
                true, closeAfter2sec);
    }   

    /**
     * Charge une image
     */
    public static ImageIcon getImageIcon(String name)
    {
    	Toolkit toolkit = Toolkit.getDefaultToolkit();
    	
        URL url;

        if (hashtable.containsKey(name) == true)
            return (ImageIcon) hashtable.get(name);

        ImageIcon icon = new ImageIcon(toolkit.getImage(AnalyseFrame.class.getResource("images/" + name)));
        
        hashtable.put(name, icon);
        
        return icon;
        
        /*url = ClassLoader.getSystemResource("org/analyse/core/gui/images/" + name);
        if (url == null) {
            url = ClassLoader.getSystemResource("org/analyse/core/gui/images/Home.png");
        }

        ImageIcon icon = new ImageIcon(url);

        hashtable.put(name, icon);

        return icon;*/
    }
	/**
	 * Ouvre une police de caractres  partir d'un fichier TrueType.
	 * Ces polices sont stockes dans le package org.fia.fonts.<br>
	 * <i>Ne pas utiliser</i>
	 * @param name nom de la police de caractres (ex : americana.ttf).
	 * @return la police demande.
	 * @see getFont(String name)
	 */
	public static Font openFont(String name)
	{
		if(fonts.containsKey(name))
			return (Font) fonts.get(name);
		
		Font font = null;
		InputStream is = ClassLoader.getSystemResourceAsStream("org/analyse/core/gui/fonts/" + name);
		
		if (is == null)
		{
			System.err.println("Cannot open font " + name + ".");
			System.exit(1);
		}
		
		try {
			 font = Font.createFont(Font.TRUETYPE_FONT, is);
		} catch (Exception e) {
			System.err.println("Cannot create font " + name + ".");
			System.exit(1);
		}
		return font;
	}
	
	/**
	 * Retourne une police de caractres. Par dfaut, la taille est de 12.
	 * @param name nom de la police de caractres.
	 * @return la police demande.
	 * @see openFont(String name)
	 */
	public static Font getFont(String name)
	{
		return getFont(name, Font.PLAIN, 12);
	}
	
	/**
	 * Retourne une police de caractres avec une taille demande.
	 * @param name nom de la police de caractres.
	 * @param size taille de la police de caractres.
	 * @return la police demande.
	 * @see openFont(String name)
	 */
	public static Font getFont(String name, float size)
	{
		return getFont(name, Font.PLAIN, size);
	}
	
	/**
	 * Retourne une police de caractres avec une taille et 
	 * un style demands.
	 * @param name nom de la police de caractres.
	 * @param style taille de la police de caractres.
	 * @param size style de la police de caractres.
	 * @return la police demande.
	 * @see openFont(String name)
	 */
	public static Font getFont(String name, int style, float size)
	{
		return openFont(name).deriveFont(style, size);
	}
}