/*
 * 10/21/2003 - 16:34:49
 *
 * AnalyseSave.java - 
 * Copyright (C) 2003 Dreux Loic
 * dreuxl@free.fr
 *  
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.analyse.core.util.save;

import java.io.File;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.analyse.core.gui.AnalyseFrame;
import org.analyse.core.modules.AnalyseModule;
import org.analyse.core.save.FiltreASI;
import org.analyse.core.util.*;
import org.analyse.main.Main;

/**
 * Cette class contient toute les mthodes pour grer la sauvegarde des donnes :
 * boite de dialogue, gestion des diffrents filtres de sauvegardes, ...
 */
public class AnalyseSave
{
    /** This constant defines an open dialog box. */
    public static final int OPEN = 0;

    /** This constant defines a save dialog box. */
    public static final int SAVE = 1;

    /** This constant defines a save dialog box. */
    public static final int RAPPORT = 2;

    private static final Object[] options = { "Oui", "Non", "Annuler" };

    private boolean isSave;

    private String fileName;

    private JFileChooser chooser;

    private AnalyseFrame frame;

    private Vector filtres;

    public AnalyseSave(AnalyseFrame frame)
    {
        this.frame = frame;
        isSave = false;

        initFilter();
        initFileChooser();
    }
    
    public void closeProgram()
    {
    	if(GUIUtilities.question("Voulez-vous sauvegarder le fichier en cours ?"))
    		save();
    }

    /**
     * Sauvegarde les donnes dans un fichier, si le document  dj t
     * sauvegard une fois, le nom du fichier ne change pas, sinon, la mthode
     * saveAs est lance.
     */
    public void save()
    {
        if (isSave == false) {
            saveAs();
            return;
        }

        isSave = true;
        AnalyseFilter af = getAnalyseFilter();
        if (af != null && af.canSave())
            ((Save) af).save(new File(this.fileName));
        frame.setTitle(Main.NAME + " - " + this.fileName);
    }

    /**
     * Sauvegarde les donnes dans un nouveau fichier, si le nouveau fichier
     * existe, celui-ci sera ecras.
     */
    public void saveAs()
    {
        String fileName;
        fileName = chooseFile(SAVE);
        if (fileName == null)
            return;

        isSave = true;

        AnalyseFilter af = getAnalyseFilter();

        if (Utilities.getExtension(fileName).equals(""))
            this.fileName = fileName + "." + af.getExtension();
        else
            this.fileName = fileName;

        if (af != null && af.canSave())
            ((Save) af).save(new File(this.fileName));
        frame.setTitle(Main.NAME + " - " + this.fileName);
    }

    /**
     * Rcupre les donnes depuis un fichier et les charge dans l'application.
     */
    public void open()
    {
        int i = JOptionPane.showOptionDialog(null,
                "Voulez-vous sauvegarder le fichier en cours ?", "Analyse",
                JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null,
                options, options[0]);

        if (i == 2)
            return;
        if (i == 0)
            save();

        String fileName = null;
        fileName = chooseFile(OPEN);

        open(fileName);
    }

    public void open(String fileName)
    {
        if (fileName == null)
            return;

        AnalyseModule mod;
        for (Enumeration e = org.analyse.main.Main.modules.elements(); e
                .hasMoreElements();) {
            mod = (AnalyseModule) e.nextElement();
            mod.clear();
        }

        isSave = true;

        this.fileName = fileName;

        AnalyseFilter af = getAnalyseFilter();
        if (af != null && af.canOpen())
            ((Open) af).open(new File(this.fileName));
        frame.setTitle(Main.NAME + " - " + this.fileName);
    }

    /**
     * Efface toutes les donnes prcdentes. Propose de sauvegarder les donnes
     * avant l'effacement.
     */
    public void clear()
    {
        int i = JOptionPane.showOptionDialog(null,
                "Voulez-vous sauvegarder le fichier en cours ?", "Analyse",
                JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null,
                options, options[0]);

        if (i == 2)
            return;
        if (i == 0)
            save();

        AnalyseModule mod;
        for (Enumeration e = org.analyse.main.Main.modules.elements(); e
                .hasMoreElements();) {
            mod = (AnalyseModule) e.nextElement();
            mod.clear();
        }

        isSave = false;
        frame.setTitle(Main.NAME + " - sans nom");
    }

    /**
     * Initialise les filtres de sauvegardes.
     */
    private void initFilter()
    {
        filtres = new Vector();
        filtres.add(new FiltreASI());
    }

    /**
     * Modifie les filtres selon le mode.
     */
    private void initFileFilter(int mode)
    {
        AnalyseFilter af;
        FileChooserFilter cf;
        chooser.resetChoosableFileFilters();
        for (Enumeration e = filtres.elements(); e.hasMoreElements();) {
            af = (AnalyseFilter) e.nextElement();
            if (mode == OPEN && af.canOpen() || mode == SAVE && af.canSave()) {
                cf = new FileChooserFilter(af.getID());
                cf.setExtension(af.getExtension());
                cf.setDescription(af.getDescription());
                chooser.addChoosableFileFilter(cf);
            }
        }

    }

    /**
     * Retourne l'objet <code>AnalyseFilter</code> correspondant  la
     * description du FileChooserFilter courant.
     */
    private AnalyseFilter getAnalyseFilter()
    {
        AnalyseFilter af;
        String ext = "";
        String ID = "";
        FileChooserFilter ff;

        if (chooser.getFileFilter() instanceof FileChooserFilter) {
            ff = (FileChooserFilter) (chooser.getFileFilter());
            ID = ff.getID();
        }
        if (fileName != null)
            ext = Utilities.getExtension(fileName);

        for (Enumeration e = filtres.elements(); e.hasMoreElements();) {
            af = (AnalyseFilter) e.nextElement();
            if (af.getID().equals(ID) || af.getExtension().equals(ext))
                return af;
        }

        return null;
    }

    /**
     * Initialise la boite de dialogue <code>JFileChooser</code>.
     */
    private void initFileChooser()
    {
        chooser = new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        chooser.setFileHidingEnabled(true);
    }

    /**
     * Retourne la boite de dialogue <code>JFileChooser</code>.
     * 
     * @param mode
     *            param?tre indiquant le type de boite : ouverture ou
     *            sauvegarde.
     */
    private JFileChooser getFileChooser(int mode)
    {
        initFileFilter(mode);
        if (mode == SAVE || mode == RAPPORT) {
            chooser.setDialogType(JFileChooser.SAVE_DIALOG);
        } else {
            chooser.setDialogType(JFileChooser.OPEN_DIALOG);
        }

        return chooser;
    }

    private String chooseFile(int mode)
    {
        JFileChooser chooser = getFileChooser(mode);

        if (chooser.showDialog(org.analyse.main.Main.analyseFrame, null) == JFileChooser.APPROVE_OPTION) {
            return chooser.getSelectedFile().getAbsolutePath();
        }

        return null;
    }
}