/*
 * 06/04/2003 - 13:56:06
 * 
 * MCDPanel.java - Copyright (C) 2003 Dreux Loic dreuxl@free.fr
 * 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.analyse.merise.gui.panel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.stream.FileImageOutputStream;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import org.analyse.core.gui.action.BasicAction;
import org.analyse.core.modules.AnalysePanel;
import org.analyse.core.util.GUIUtilities;
import org.analyse.core.util.Utilities;
import org.analyse.core.util.save.AnalyseFilter;
import org.analyse.core.util.save.FileChooserFilter;
import org.analyse.main.Main;
import org.analyse.merise.gui.dialog.EntiteDialog;
import org.analyse.merise.gui.dialog.LienDialog;
import org.analyse.merise.gui.table.DictionnaireTable;
import org.analyse.merise.mcd.composant.MCDAssociation;
import org.analyse.merise.mcd.composant.MCDComponent;
import org.analyse.merise.mcd.composant.MCDEntite;
import org.analyse.merise.mcd.composant.MCDLien;
import org.analyse.merise.mcd.composant.MCDObjet;
import org.analyse.merise.mcd.composant.MLDComponent;
import org.analyse.merise.sql.SQLCommand;

import com.sun.imageio.plugins.png.PNGImageWriter;

public class MCDPanel extends AnalysePanel
{
    private ActionHandler actionHandler;

    private BasicAction addEntite, addAssociation, addLien;

    private BasicAction delLien, delObjet;

    private BasicAction modEntite, modAssociation, modLien;

    private BasicAction verif, buildMLD;

    private BasicAction saveGraphic;

    private MCDComponent mcdComponent;

    private MLDComponent mldComponent;

    private SQLCommand sqlCommand;

    private JPanel toolbar;

    private JPopupMenu popupLien, popupEntite, popupAssociation,
            popupSaveGraphic;

    private JFileChooser chooser;

    private JToggleButton btnLien;

    private EntiteDialog entiteDialog;

    private LienDialog lienDialog;

    private MCDObjet objet;

    private MCDLien lien;

    public MCDPanel(MCDComponent mcdComponent, MLDComponent mldComponent,
            SQLCommand sqlCommand)
    {
        super("MCD");
        
        this.mcdComponent = mcdComponent;
        this.mldComponent = mldComponent;
        this.sqlCommand = sqlCommand;
        this.actionHandler = new ActionHandler();

        initAction();
        initToolbar();
        initPopup();
        initDialog();

        chooser = new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        chooser.setFileHidingEnabled(true);

        AnalyseFilter af;
        FileChooserFilter cf;
        chooser.resetChoosableFileFilters();
        cf = new FileChooserFilter("PNG");
        cf.setExtension("png");
        cf.setDescription("Image PNG");
        chooser.addChoosableFileFilter(cf);
        chooser.setDialogType(JFileChooser.SAVE_DIALOG);

        this.setBorder(BorderFactory.createEmptyBorder(3, 3, 0, 3));

        this.setLayout(new BorderLayout());
        this.add(BorderLayout.CENTER, new JScrollPane(mcdComponent));
        this.add(BorderLayout.NORTH, toolbar);
        
        mcdComponent.addMouseListener(new MouseHandler());
    }

    private void initToolbar()
    {
        toolbar = new JPanel();
        toolbar.setLayout(new FlowLayout(FlowLayout.LEFT));

        toolbar.add(new JButton(addEntite) {

            {
                setText("");
                addMouseListener(Main.statusbar.getHandler());
            }
        });

        toolbar.add(new JButton(addAssociation) {

            {
                setText("");
                addMouseListener(Main.statusbar.getHandler());
            }
        });

        toolbar.add(btnLien = new JToggleButton(addLien) {

            {
                setText("");
                addMouseListener(Main.statusbar.getHandler());
            }
        });

        toolbar.add(new JToolBar.Separator());

        toolbar.add(new JButton(saveGraphic) {

            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });
    }

    private void initPopup()
    {
        popupLien = new JPopupMenu();

        popupLien.add(new JMenuItem(modLien) {

            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });

        popupLien.addSeparator();

        popupLien.add(new JMenuItem(delLien) {

            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });

        popupEntite = new JPopupMenu();

        popupEntite.add(new JMenuItem(modEntite) {

            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });

        popupEntite.addSeparator();

        popupEntite.add(new JMenuItem(delObjet) {

            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });

        popupAssociation = new JPopupMenu();

        popupAssociation.add(new JMenuItem(modAssociation) {

            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });

        popupAssociation.addSeparator();

        popupAssociation.add(new JMenuItem(delObjet) {

            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });

        popupSaveGraphic = new JPopupMenu();
        popupSaveGraphic.add(new JMenuItem(saveGraphic) {
            {
                addMouseListener(Main.statusbar.getHandler());
            }
        });
    }

    private void initAction()
    {        
        addEntite = new BasicAction("Entit", "Ajouter une entit", "ADD_ENT",
                GUIUtilities.getImageIcon("Entite.png"), 0, null);
        addEntite.addActionListener(actionHandler);

        addAssociation = new BasicAction("Association",
                "Ajouter une association", "ADD_ASS", GUIUtilities
                        .getImageIcon("Association.png"), 0, null);
        addAssociation.addActionListener(actionHandler);

        addLien = new BasicAction("Lien", "Ajouter un lien", "ADD_LIEN",
                GUIUtilities.getImageIcon("Lien.png"), 0, null);
        addLien.addActionListener(actionHandler);

        delLien = new BasicAction("Supprimer", "Supprimer le lien", "DEL_LIEN",
                GUIUtilities.getImageIcon("Delete.png"), 0, null);
        delLien.addActionListener(actionHandler);

        delObjet = new BasicAction("Supprimer", "Supprimer l'objet",
                "DEL_OBJET", GUIUtilities.getImageIcon("Delete.png"), 0, null);
        delObjet.addActionListener(actionHandler);

        modEntite = new BasicAction("Modifier", "Modifier l'entit",
                "MOD_OBJET", GUIUtilities.getImageIcon("Edit.png"), 0, null);
        modEntite.addActionListener(actionHandler);

        modAssociation = new BasicAction("Modifier", "Modifier l'association",
                "MOD_OBJET", GUIUtilities.getImageIcon("Edit.png"), 0, null);
        modAssociation.addActionListener(actionHandler);

        modLien = new BasicAction("Modifier", "Modifier le lien", "MOD_LIEN",
                GUIUtilities.getImageIcon("Edit.png"), 0, null);
        modLien.addActionListener(actionHandler);

        verif = new BasicAction("Vrification", "Vrification du MCD",
                "VERIF_MCD", GUIUtilities.getImageIcon("OK.png"), 0, null);
        verif.addActionListener(actionHandler);

        buildMLD = new BasicAction("Construction",
                "Construction du MLD et du script SQL", "BUILD_MLD",
                GUIUtilities.getImageIcon("Build.png"), 0, KeyStroke
                        .getKeyStroke(KeyEvent.VK_F5, 0));
        buildMLD.addActionListener(actionHandler);

        saveGraphic = new BasicAction("Sauvegarder le graphique",
                "Sauvegarder le graphique dans un fichier PNG", "SAVE_GRAPH",
                GUIUtilities.getImageIcon("Save.png"), 0, null);
        saveGraphic.addActionListener(actionHandler);
    }

    private String chooseFile()
    {
        if (chooser.showDialog(org.analyse.main.Main.analyseFrame, null) == JFileChooser.APPROVE_OPTION) {
            return chooser.getSelectedFile().getAbsolutePath();
        }

        return null;
    }

    public BasicAction getVerif()
    {
        return verif;
    }

    public BasicAction getBuildLMD()
    {
        return buildMLD;
    }

    private void initDialog()
    {
        entiteDialog = new EntiteDialog(mcdComponent.getData());
        lienDialog = new LienDialog();
    }

    private class ActionHandler implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            String action = e.getActionCommand();

            if (action.equals("ADD_ENT")) {
                DictionnaireTable data = mcdComponent.getData();
                data.addObserver(mcdComponent.addEntite());
            } else if (action.equals("ADD_ASS")) {
                DictionnaireTable data = mcdComponent.getData();
                data.addObserver(mcdComponent.addAssociation());
            } else if (action.equals("ADD_LIEN")) {
                if (btnLien.getSelectedObjects() != null)
                    mcdComponent.addLien();
                else
                    mcdComponent.annulerCreerLien();
            } else if (action.equals("DEL_LIEN")) {
                String mess = "Voulez-vous vraiment supprimer le lien slectionn ?";
                if (JOptionPane.showConfirmDialog(null, mess, "Analyse",
                        JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION)
                    return;
                mcdComponent.removeLien();
            } else if (action.equals("DEL_OBJET")) {
                DictionnaireTable data = mcdComponent.getData();
                String mess = "Voulez-vous vraiment supprimer l'objet slectionn ?";
                if (JOptionPane.showConfirmDialog(null, mess, "Analyse",
                        JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION)
                    return;
                data.deleteObserver(mcdComponent.removeObjet());
            } else if (action.equals("MOD_OBJET")) {
                entiteDialog.load((MCDObjet) objet);
            } else if (action.equals("MOD_LIEN")) {
                lienDialog.load((MCDLien) lien);
            } else if (action.equals("VERIF_MCD")) {
                mcdComponent.isCorrect(MCDComponent.SHOW_ALL);
            } else if (action.equals("BUILD_MLD")) {
                if (mcdComponent.buildMLD(mldComponent,
                        MCDComponent.HIDE_ALL)) {
                    mldComponent.buildSQL(mcdComponent.getData(), sqlCommand);
                }
                mcdComponent.buildMLD(mldComponent,
                        MCDComponent.CREATE_MCD);
            } else if (action.equals("SAVE_GRAPH")) {
                String fileName = chooseFile();
                if (fileName == null)
                    return;

                if (!Utilities.getExtension(fileName).equals("png")
                        && !Utilities.getExtension(fileName).equals("PNG"))
                    fileName = fileName + ".png";

                mcdComponent.enleverFocus();

                try {
                    File imageFile;
                    FileImageOutputStream outputStream;
                    BufferedImage img;
                    Graphics g;
                    Graphics2D g2d;
                    Rectangle2D r;

                    imageFile = new File(fileName);
                    img = new BufferedImage(
                            (int) (mcdComponent.getPreferredSize().getWidth()),
                            (int) (mcdComponent.getPreferredSize().getHeight()),
                            BufferedImage.TYPE_INT_RGB);

                    g = img.getGraphics();
                    g2d = (Graphics2D) g;

                    g2d.setColor(new Color(255, 255, 255));
                    r = new Rectangle2D.Double(0, 0, img.getWidth(), img
                            .getHeight());
                    g2d.fill(r);

                    mcdComponent.paintComponent(g2d);

                    PNGImageWriter writer = new PNGImageWriter(null);

                    writer.setOutput(outputStream = new FileImageOutputStream(
                            imageFile));
                    writer.write(img);

                    outputStream.close();

                    writer.dispose();

                } catch (IOException err) {
                    GUIUtilities.error("Impossible de sauvegarder le fichier "
                            + fileName);
                }
            }
        }
    }

    private class MouseHandler extends MouseAdapter
    {
        public void mouseReleased(MouseEvent e)
        {
            objet = null;
            lien = null;
            //if (e.isPopupTrigger()) Ne marche pas avec le JDK d'IBM
            if (e.getButton() == MouseEvent.BUTTON3) {
                Object o = mcdComponent.getObjectFromLocation(e.getX(), e
                        .getY());
                if (o instanceof MCDLien) {
                    popupLien.show(e.getComponent(), e.getX(), e.getY());
                    lien = (MCDLien) o;
                } else if (o instanceof MCDAssociation) {
                    popupAssociation.show(e.getComponent(), e.getX(), e.getY());
                    objet = (MCDObjet) o;
                } else if (o instanceof MCDEntite) {
                    popupEntite.show(e.getComponent(), e.getX(), e.getY());
                    objet = (MCDObjet) o;
                } else {
                    popupSaveGraphic.show(e.getComponent(), e.getX(), e.getY());
                }
            }
            if (btnLien.getSelectedObjects() != null)
                btnLien.doClick();
        }

        public void mouseClicked(MouseEvent e)
        {
            if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() == 2) {
                Object o = mcdComponent.getObjectFromLocation(e.getX(), e
                        .getY());
                if (o instanceof MCDLien) {
                    lienDialog.load((MCDLien) o);
                } else if (o instanceof MCDAssociation) {
                    entiteDialog.load((MCDAssociation) o);
                } else if (o instanceof MCDEntite) {
                    entiteDialog.load((MCDObjet) o);
                }
            }
        }
    }
}