/*
 * 05/23/2003 - 16:41:45
 *
 * DictionnaireTable.java -
 * Copyright (C) 2003 Dreux Loic
 * dreuxl@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.analyse.merise.gui.table;

import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import org.analyse.core.util.Utilities;

/**
 * Cette table contient les informations utilisables par le MCD.
 */
public class DictionnaireTable extends AbstractTableModel
{
    public static final int UP = 0;

    public static final int DOWN = 1;

    public static final int NAME = 0;

    public static final int ID = 1;

    public static final int TYPE = 2;

    public static final int SIZE = 3;

    public static final int USE = 4;

    /** Nom des 5 colonnes. */
    private final String[] columnNames = { "Nom", "ID", "Type", "Taille",
            "Utilis" };

    /** Donnes contenu dans la table. */
    private Object[][] data;

    /** Nombre de lignes saisies dans la table. */
    private int sizeRow;

    /** Diffrents types de donnes possibles. */
    private Vector types;

    private Observable observable;

    /**
     * Crer une nouvelle <code>DictionnaireTable</code>.
     *  
     */
    public DictionnaireTable(Vector types)
    {
        this.types = types;

        data = new Object[0][5];

        sizeRow = 0;
        addNewLine();

        observable = new Observable() {
            public void notifyObservers()
            {
                setChanged();
                super.notifyObservers();
            }

            public void notifyObservers(Object arg)
            {
                setChanged();
                super.notifyObservers(arg);
            }
        };
    }

    /**
     * Ajoute un observateur sur la table. Utilis par les classes MCDObject.
     */
    public void addObserver(Observer obs)
    {
        observable.addObserver(obs);
    }

    /**
     * Supprime un observateur de la table.
     */
    public void deleteObserver(Observer obs)
    {
        observable.deleteObserver(obs);
    }

    /**
     * Ajoute une ligne vide dans le tableau
     */
    public void addNewLine()
    {
        sizeRow++;

        if (sizeRow >= data.length) {
            Object[][] dataSv = new Object[sizeRow + 10][5];
            ;

            for (int i = 0; i < sizeRow - 1; i++)
                dataSv[i] = data[i];

            this.data = dataSv;
        }

        data[sizeRow - 1][0] = "";
        data[sizeRow - 1][1] = "";
        data[sizeRow - 1][2] = types.elementAt(0);
        data[sizeRow - 1][3] = new Integer(0);
        data[sizeRow - 1][4] = new Boolean(false);
        fireTableDataChanged();
    }

    /**
     * Ajoute une nouvelle donne dans le tableau
     */
    public void addData(String code, String nom, String type, String taille,
            String utilise)
    {
        data[sizeRow - 1][0] = nom;
        data[sizeRow - 1][1] = code;
        data[sizeRow - 1][2] = type;
        data[sizeRow - 1][3] = new Integer(taille);
        data[sizeRow - 1][4] = new Boolean(utilise);
        addNewLine();
    }

    public void addData(String nom, String type, String taille)
    {
        Integer tailleInt;

        try {
            tailleInt = new Integer(taille);
        } catch (NumberFormatException e) {
            tailleInt = new Integer(0);
        }

        if (!contains(Utilities.normaliseString(nom, Utilities.LOWER))) {
            data[sizeRow - 1][0] = nom;
            data[sizeRow - 1][1] = Utilities.normaliseString(nom,
                    Utilities.LOWER);
            data[sizeRow - 1][2] = type;
            data[sizeRow - 1][3] = tailleInt;
            data[sizeRow - 1][4] = new Boolean(false);
            addNewLine();
        }
    }

    /**
     * Supprime plusieurs lignes.
     */
    public void delLines(boolean[] supp)
    {
        int indiceReel = 0;
        for (int i = 0; i < supp.length; i++) {
            if (supp[i]) {
                observable.notifyObservers(new ArgObserverTable(
                        ArgObserverTable.DELETE, (String) data[indiceReel][1]));
                swapLines(indiceReel + 1, sizeRow - 1, UP);
                sizeRow--;
            } else {
                indiceReel++;
            }
        }

        //Actualise le tableau
        fireTableDataChanged();
    }

    /**
     * Dplace une srie de ligne dans la direction demande.
     * 
     * @param start
     *            dbut de la slection
     * @param end
     *            fin de la slection
     * @param direction
     *            direction bas ou haut
     */
    public boolean swapLines(int start, int end, int direction)
    {
        if (start == 0 && direction == UP || end >= sizeRow - 2
                && direction == DOWN || start > end)
            return false;

        if (direction == DOWN)
            for (int i = end; i >= start; i--)
                swapLines(i, i + 1);
        else
            //if(direction == UP)

            for (int i = start; i <= end; i++)
                swapLines(i, i - 1);

        //Actualise le tableau
        fireTableDataChanged();
        return true;
    }

    /**
     * Permute deux lignes
     */
    private void swapLines(int first, int second)
    {
        Object[] line;

        line = data[first];
        data[first] = data[second];
        data[second] = line;
    }

    /**
     * Indique si une donne est prsente dans la table
     * 
     * @param code
     *            code de la donne.
     */
    public boolean contains(String code)
    {
        for (int i = 0; i < sizeRow; i++)
            if (code.equals((String) (data[i][1])))
                return true;
        return false;
    }

    /**
     * Retourne l'identifiant d'une information
     * 
     * @param i
     *            index du code ? retourner
     */
    public String getID(int i)
    {
        if (i < sizeRow)
            return (String) data[i][1];
        return null;
    }

    /**
     * Retourne la valeur d'une case selon l'identifiant et la columns.
     */
    public Object getValue(String ID, int col)
    {
        return getValueAt(getIndex(ID), col);
    }

    /**
     * Retourne la valeur d'une case.
     * 
     * @param row
     *            ligne de la case
     * @param col
     *            colone de la case
     */
    public Object getValueAt(int row, int col)
    {
        return data[row][col];
    }

    /**
     * Indique si une donne est utilis dans le MCD.
     */
    public boolean getUse(int row)
    {
        return ((Boolean) data[row][USE]).booleanValue();
    }

    /**
     * Modifie la valeur USE dans la table. <br>
     * Utilis par le MCD
     */
    public void setUse(String ID, boolean use)
    {
        int row = getIndex(ID);
        if (row == -1)
            return;

        data[row][USE] = new Boolean(use);
    }

    /**
     * 
     * Retourne le numro de ligne d'une donne.
     * 
     * @param ID
     *            identifiant de la donne
     */
    public int getIndex(String ID)
    {
        for (int i = 0; i < sizeRow; i++)
            if (ID.equals((String) (data[i][1])))
                return i;
        return -1;
    }

    /**
     * Vrifie que toutes les informations sont dans le MCD.
     */
    public boolean allUse()
    {
        for (int i = 0; i < sizeRow - 1; i++)
            if (!((Boolean) data[i][4]).booleanValue())
                return false;
        return true;
    }
    
    /**
     * Retourne le nombre de colonnes.
     */
    public int getColumnCount()
    {
        return columnNames.length;
    }

    /**
     * Retourne le nombre de lignes.
     */
    public int getRowCount()
    {
        return sizeRow;
    }

    /**
     * Retourne le nom d'une colonne.
     * 
     * @param col
     *            index de la colonne
     */
    public String getColumnName(int col)
    {
        return columnNames[col];
    }

    /**
     * Retourne le type d'une information selon la colonne.
     * 
     * @param col
     *            colonne
     */
    public Class getColumnClass(int col)
    {
        return getValueAt(0, col).getClass();
    }

    /**
     * Retourne les diffrents types de donnes.
     */
    public Vector getTypes()
    {
        return types;
    }
    
    public boolean verifySize(int i)
    {
    	int size = ((Integer)data[i][DictionnaireTable.SIZE]).intValue();
    	
    	if(data[i][DictionnaireTable.TYPE].equals("varchar") && !(size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("int") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("uniqueidentifier") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("bigint") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("bit") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("datetime") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("int") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("float") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("tinyint") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("date") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("time") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("bool") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("int2") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("int4") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("int8") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("float4") && (size > 0))
    		return false;
    	if(data[i][DictionnaireTable.TYPE].equals("float8") && (size > 0))
    		return false;
    	
    	return true;
    }

    /**
     * Indique si une case est ditable ou pas.
     */
    public boolean isCellEditable(int row, int col)
    {
        if (col == ID || col == USE)
            return false;
        return true;

    }

    /**
     * Modifie une cellule.
     */
    public void setValueAt(Object value, int row, int col)
    {
        // Sauvegarde l'ancien identifiant.
        String oldID = (String) data[row][1];

        // Vrifie que l'identifiant n'existe pas dj.
        if ((col == NAME
                && contains(Utilities.normaliseString((String) value,
                        Utilities.LOWER)) && getIndex(Utilities
                .normaliseString((String) value, Utilities.LOWER)) != row)
                || (col == NAME && value.equals(""))
                || (!isCellEditable(row, col)))
            return;

        // Transforme la chaine de caractre en Integer
        if (data[0][col] instanceof Integer && value instanceof String) {
            try {
                data[row][col] = new Integer(value.toString());
            } catch (NumberFormatException e) {
            }
        } else {
            data[row][col] = value;

            if (col == NAME) {
                data[row][ID] = Utilities.normaliseString(
                        (String) (data[row][NAME]), Utilities.LOWER);
                observable
                        .notifyObservers(new ArgObserverTable(
                                ArgObserverTable.RENAME, oldID,
                                (String) data[row][ID]));
                fireTableCellUpdated(row, ID);
            }

        }
        // Ajoute une ligne si row
        if (row == getRowCount() - 1 && data[row][NAME] != null)
            addNewLine();
        fireTableCellUpdated(row, col);
    }

    /**
     * Vide le tableau
     */
    public void clear()
    {
        data = new Object[0][5];
        sizeRow = 0;
        addNewLine();

        observable.deleteObservers();

        fireTableDataChanged();
    }
}