/*
 * 02/25/2002 - 13:26:10
 *
 * MCD
 * Copyright (C) 2002 Dreux Loic
 * dreuxl@free.fr
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.analyse.merise.mcd.composant;

import java.awt.Point;
import java.util.Enumeration;
import java.util.Observer;

import org.analyse.core.gui.zgraph.ZElement;
import org.analyse.core.gui.zgraph.ZGraphique;
import org.analyse.core.gui.zgraph.ZLien;
import org.analyse.core.util.GUIUtilities;
import org.analyse.core.util.Utilities;
import org.analyse.merise.gui.table.DictionnaireTable;

/**
 * Composant Graphique reprsentant un MCD.
 */
public class MCDComponent extends ZGraphique implements Observer {
	public final static int SHOW_ALL = 0;

	public final static int SHOW_ERRORS = 1;

	public final static int HIDE_ALL = 2;

	public final static int CREATE_MCD = 3;

	public final static int PORTEUSE = 0;

	public final static int UN_UN = 1;

	public final static int UN_N = 2;

	private boolean newLink = false;

	/** Premier objet lors de la cration d'un lien. */
	private MCDObjet object1 = null;

	/** Deuxi?me objet lors de la cration d'un lien. */
	private MCDObjet object2 = null;

	/** Pour la position du lien */
	private int x1 = -1, y1 = -1, x2 = -1, y2 = -1;

	/** dictionnaire des informations */
	private DictionnaireTable data;

	private String msg;

	/**
	 * Crer un nouveau <code>MCDComponent</code>
	 * 
	 * @param data
	 *            Dictionnaire des informations  associer au MCD.
	 */
	public MCDComponent(DictionnaireTable data) {
		super();
		this.data = data;

		data.addObserver(this);
	}

	/**
	 * Ajoute un MCDObjet dans le MCD.
	 * 
	 * Utilis lors d'un chargement d'un fichier
	 */
	public void addObjet(MCDObjet obj) {
		data.addObserver(obj);
		super.addElement(obj);
	}

	/**
	 * Ajoute une nouvelle <code>MCDEntite</code> vide.
	 * 
	 * Utilis lors de la cration d'une nouvelle entit
	 */
	public MCDEntite addEntite() {
		MCDEntite ent;
		super.addElement(ent = new MCDEntite(this));
		data.addObserver(ent);
		return ent;
	}

	/**
	 * Ajoute une nouvelle <code>MCDAssociation</code> vide.
	 * 
	 * Utilis lors de la cration d'une nouvelle association
	 */
	public MCDAssociation addAssociation() {
		MCDAssociation ass;
		super.addElement(ass = new MCDAssociation(this));
		data.addObserver(ass);
		return ass;
	}

	/**
	 * Appelle la mthode creerLien, cette mthode permet  l'utilisateur de
	 * slectionner deux <code>MCDObjet</code> afin de les lier.
	 * 
	 * Utilis lors de la cration d'un nouveau lien
	 */
	public void addLien() {
		creerLien(new MCDLien());
	}

	/**
	 * Supprime un lien ainsi que leurs rfrences dans les
	 * <code>MCDObjet</code>.
	 */
	public void removeLien(ZLien lien) {
		MCDObjet elem1 = (MCDObjet) (lien.getElement(ZLien.ELEM1));
		elem1.delLink((MCDLien) lien);
		MCDObjet elem2 = (MCDObjet) (lien.getElement(ZLien.ELEM2));
		elem2.delLink((MCDLien) lien);
		super.removeLien(lien);
	}

	/**
	 * Supprime le lien slectionn
	 */
	public void removeLien() {
		removeLien(lienClic);
	}

	/**
	 * Supprime le <code>MCDObjet</code>. Cette mthode s'occupe de librer
	 * les informations que contenait le MCDObjet.
	 */
	public void removeObjet(ZElement element) {
		MCDObjet obj = (MCDObjet) element;
		obj.clearInformations();
		super.removeElement(element);
	}

	/**
	 * Supprime le <code>MCDObjet</code> slectionn.
	 */
	public MCDObjet removeObjet() {
		ZElement element = elementClic;
		this.removeObjet(elementClic);
		return (MCDObjet) element;
	}

	/**
	 *  
	 */
	public void clear() {
		super.clearAll();
	}

	/**
	 * Retourne le <code>MCDObjet</code> correspondant au nom pass en
	 * paramtre.
	 */
	public MCDObjet getElement(String name) {
		for (Enumeration e = enumElements(); e.hasMoreElements();) {
			MCDObjet o = (MCDObjet) e.nextElement();
			if (o.getName().equals(name))
				return o;
		}
		return null;
	}

	/**
	 * Indique si il est possible de crer un lien entre les deux lments. <br>
	 * Retourne vrai si les deux lments reprsente une association et une
	 * entit, faux dans le cas contraire.
	 */
	public boolean peutCreerLien(ZElement elem1, ZElement elem2) {
		return super.peutCreerLien(elem1, elem2)
				&& (elem1 instanceof MCDAssociation
						&& elem2 instanceof MCDEntite || elem2 instanceof MCDAssociation
						&& elem1 instanceof MCDEntite);
	}

	/**
	 * Retourne la table du dictionnaire des informations correspondant au MCD.
	 */
	public DictionnaireTable getData() {
		return data;
	}

	/**
	 * Retourne vrai si le MCD est correct.
	 */
	public boolean isCorrect(int showErrors) {
		MCDObjet obj;
		boolean correct = true;
		boolean dicoCorrect = true;
		msg = "";

		// Vrification du dictionnaire
		if (!data.allUse())
			msg += "<b style=\"color: blue;\">/!\\Attention</b><br>Des informations du dictionnaire ne sont pas dans le mcd<br>";

		for (int i = 0; i < data.getRowCount(); i++) {
			if(!data.verifySize(i))
			{
				dicoCorrect = false;
				msg += "<b style=\"color: red;\">/!\\Attention</b><br>La taille de l'information "
						+ (String) data.getValueAt(i, DictionnaireTable.NAME)
						+ " est incorrect<br>";
			}
		}

		if (dicoCorrect) {
			msg += "Le dictionnaire des informations ne contient pas d'erreurs<br>";
		} else {
			msg += "Le dictionnaire des informations contient des erreurs, veuillez les corriger avant de gnrer le MLD.<br>";
		}
		//-----------------------------

		if (dicoCorrect) {
			for (Enumeration e = this.elementsZElements(); e.hasMoreElements();) {
				obj = (MCDObjet) e.nextElement();

				if (obj instanceof MCDAssociation) {
					if (!correct)
						associationCorrect((MCDAssociation) obj);
					else
						correct = associationCorrect((MCDAssociation) obj);
				} else if (obj instanceof MCDEntite) {
					if (((MCDEntite) obj).sizeInformation() < 1) {
						correct = false;
						msg += "<b style=\"color: red;\">/!\\</b> L'entit \""
								+ obj.getName()
								+ "\" n'a pas d'identifiant.<br>";
					}
				}
			}
		} else {
			correct = false;
		}

		if (correct && (showErrors == SHOW_ALL)) {
			GUIUtilities.messageHTML(msg
					+ "<b style=\"color: blue;\">MCD Correct.</b>", true, 300, 250);
		} else if ((!correct)
				&& (showErrors == SHOW_ERRORS || showErrors == SHOW_ALL)) {
			GUIUtilities.messageHTML(msg
					+ "<b style=\"color: red;\">MCD Incorrect.</b>", false);
		} else if (correct && showErrors == CREATE_MCD) {
			GUIUtilities.messageHTML(msg
					+ "<b style=\"color: blue;\">Cration du MLD russi.</b>",
					true, 300, 250);
		} else if (!correct && showErrors == CREATE_MCD) {
			GUIUtilities
					.messageHTML(msg
							+ "<b style=\"color: red;\">MCD Incorrect, impossible de crer le MLD.</b>", false);
		}

		return correct;
	}

	/**
	 * Vrifie les cardinalites des associations.
	 */
	private boolean associationCorrect(MCDAssociation ass) {
		MCDLien lien;
		boolean assMinimal1 = true;

		// L'association  plus de 2 liens
		// ou l'association est une association porteuse d'informations
		if (ass.sizeLink() > 2 || ass.sizeInformation() > 0) {
			// Vrifie que l'association a au moins deux liens dans
			// le cas de la porteuse.
			if (ass.sizeLink() < 2) {
				msg += "<b style=\"color: red;\">/!\\</b> Le nombre de liens de l'association \""
						+ ass.getName() + "\" est infrieur  deux.<br>";
				return false;
			}

			// Vrifie que les cardinalits de chaque liens est  N
			for (Enumeration e = ass.links(); e.hasMoreElements();) {
				lien = (MCDLien) e.nextElement();

				if (!lien.getCardMax().equals("N")) {
					msg += "<b style=\"color: red;\">/!\\</b> Les cardinalits maximales des liens de l'association \""
							+ ass.getName() + "\" sont incorrectes.<br>";
					return false;
				}
				// Vrifie que toutes les cardinalits minimales ne sont
				// pas  1.
				if (!lien.getCardMin().equals("1"))
					assMinimal1 = false;
			}
		} else if (ass.sizeLink() == 2) {
			// Vrifie que toutes les cardinalits minimales ne sont
			// pas  1.
			/*for (Enumeration e = ass.links(); e.hasMoreElements();) {
				lien = (MCDLien) e.nextElement();
				if (!lien.getCardMin().equals("1"))
					assMinimal1 = false;
			}*/
		} else {
			msg += "<b style=\"color: red;\">/!\\</b> Le nombre de liens de l'association \""
					+ ass.getName() + "\" est infrieur  deux.<br>";
			return false;
		}

		// Vrifie que toutes les cardinalits minimales ne sont
		// pas  1.
		/*if (assMinimal1 == true) {
			msg += "<b style=\"color: red;\">/!\\</b> Les cardinalits minimales de l'association \""
					+ ass.getName() + "\" sont toutes  1.<br>";
			return false;
		}*/

		return true;
	}

	/**
	 * Retourne vrai lorsque une nouvelle table va etre cre  partir de
	 * l'association.
	 */
	private boolean newTable(MCDAssociation ass) {
		if (ass.sizeLink() > 2 || ass.sizeInformation() > 0)
			return true;

		boolean porteuse = true;
		for (Enumeration e = ass.links(); e.hasMoreElements();)
			if (!((MCDLien) e.nextElement()).getCardMax().equals("N"))
				porteuse = false;

		return porteuse;
	}

	/**
	 * Retourne le type de l'association
	 */
	private int typeAssociation(MCDAssociation ass) {
		if (ass.sizeLink() > 2 || ass.sizeInformation() > 0)
			return PORTEUSE;

		boolean porteuse = true;
		boolean un_un = true;
		for (Enumeration e = ass.links(); e.hasMoreElements();) {
			MCDLien lien = (MCDLien) e.nextElement();
			if (!lien.getCardMax().equals("N"))
				porteuse = false;
			if (!lien.getCardMax().equals("1"))
				un_un = false;

		}

		if (porteuse)
			return PORTEUSE;
		if (un_un)
			return UN_UN;
		return UN_N;
	}

	/**
	 * Retourne l'objet <code>MCDEntite</code> li  l'association
	 * 
	 * @param ass
	 *            association
	 * @param card
	 *            Cardinalit
	 */
	private MCDEntite getMCDEntite(MCDAssociation ass, String card) {
		for (Enumeration e = elementsZLiens(); e.hasMoreElements();) {
			MCDLien lien = (MCDLien) (e.nextElement());

			if (lien.getElement(0).equals(ass)
					&& lien.getCardMax().equals(card))
				return (MCDEntite) (lien.getElement(1));
		}
		return null;
	}

	/**
	 * Retourne l'objet <code>MCDEntite</code> li  l'association
	 * 
	 * @param ass
	 *            association
	 * @param num
	 *            index
	 */
	private MCDEntite getMCDEntite(MCDAssociation ass, int num) {
		int tmp = 0;
		for (Enumeration e = elementsZLiens(); e.hasMoreElements();) {
			MCDLien lien = (MCDLien) (e.nextElement());

			if (lien.getElement(0).equals(ass) && tmp == num)
				return (MCDEntite) (lien.getElement(1));
			else if (lien.getElement(0).equals(ass))
				tmp++;
		}
		return null;
	}

	/**
	 * Crer le MLD correspondant au MCD.
	 */
	public boolean buildMLD(MLDComponent mld, int showErrors) {
		MLDEntite ent, ent2;
		MCDObjet obj;
		MLDLien mldLien;

		if (!this.isCorrect(showErrors))
			return false;

		mld.clearAll();
		mld.setWidth(this.getPreferredSize().getWidth());

		/*
		 * Cration des MLDEntite  partir des MCDEntite
		 */
		for (Enumeration e = this.elementsZElements(); e.hasMoreElements();) {
			obj = (MCDObjet) e.nextElement();

			if (obj instanceof MCDEntite) {
				ent = new MLDEntite(mld, Utilities.normaliseString(obj
						.getName(), Utilities.LOWER));
				ent.setPosition(new Point(obj.getX(), obj.getY()));
				ent.addIdentifiant(obj.getCodeInformation(0));
				ent.addInformations(obj.getInformations());
				mld.addMLDEntite(ent);
			}
		}

		/*
		 * Cration des MLDEntite  partir des MCDAssociation
		 */
		for (Enumeration e = this.elementsZElements(); e.hasMoreElements();) {
			obj = (MCDObjet) e.nextElement();

			if (obj instanceof MCDAssociation) {
				if (typeAssociation((MCDAssociation) obj) == PORTEUSE) {
					ent = new MLDEntite(mld, Utilities.normaliseString(obj
							.getName(), Utilities.LOWER));
					ent.setPosition(new Point(obj.getX(), obj.getY()));

					MCDLien lien;
					for (Enumeration e2 = this.elementsZLiens(); e2
							.hasMoreElements();) {
						lien = (MCDLien) e2.nextElement();
						if (lien.getMCDObjet(0).equals(obj)) {
							mldLien = new MLDLien();
							ent2 = mld.getMLDEntite(Utilities.normaliseString(
									lien.getMCDObjet(1).getName(),
									Utilities.LOWER));
							mldLien.setElement(ent, 1);
							mldLien.setElement(ent2, 0);
							mld.addLien(mldLien);

							ent.addIdentifiant(lien.getMCDObjet(1)
									.getCodeInformation(0));
							ent.addForeignKey(lien.getMCDObjet(1)
									.getCodeInformation(0), Utilities
									.normaliseString(lien.getMCDObjet(1)
											.getName(), Utilities.LOWER));
						}
					}
					ent.addInformations(obj.getInformations());

					mld.addMLDEntite(ent);
				} else if (typeAssociation((MCDAssociation) obj) == UN_UN) {
					mldLien = new MLDLien(true);

					ent = mld.getMLDEntite(Utilities.normaliseString(
							getMCDEntite((MCDAssociation) obj, 0).getName(),
							Utilities.LOWER));
					ent.addInformation(getMCDEntite((MCDAssociation) obj, 1)
							.getCodeInformation(0));
					ent.addForeignKey(getMCDEntite((MCDAssociation) obj, 1)
							.getCodeInformation(0), Utilities.normaliseString(
							getMCDEntite((MCDAssociation) obj, 1).getName(),
							Utilities.LOWER));
					mldLien.setElement(ent, 1);

					ent = mld.getMLDEntite(Utilities.normaliseString(
							getMCDEntite((MCDAssociation) obj, 1).getName(),
							Utilities.LOWER));
					ent.addInformation(getMCDEntite((MCDAssociation) obj, 0)
							.getCodeInformation(0));
					ent.addForeignKey(getMCDEntite((MCDAssociation) obj, 0)
							.getCodeInformation(0), Utilities.normaliseString(
							getMCDEntite((MCDAssociation) obj, 0).getName(),
							Utilities.LOWER));
					mldLien.setElement(ent, 0);

					mld.addLien(mldLien);
				} else if (typeAssociation((MCDAssociation) obj) == UN_N) {
					mldLien = new MLDLien();

					ent = mld.getMLDEntite(Utilities.normaliseString(
							getMCDEntite((MCDAssociation) obj, "1").getName(),
							Utilities.LOWER));
					ent.addInformation(getMCDEntite((MCDAssociation) obj, "N")
							.getCodeInformation(0));
					ent.addForeignKey(getMCDEntite((MCDAssociation) obj, "N")
							.getCodeInformation(0), Utilities.normaliseString(
							getMCDEntite((MCDAssociation) obj, "N").getName(),
							Utilities.LOWER));
					mldLien.setElement(ent, 1);

					ent = mld.getMLDEntite(Utilities.normaliseString(
							getMCDEntite((MCDAssociation) obj, "N").getName(),
							Utilities.LOWER));
					mldLien.setElement(ent, 0);

					mld.addLien(mldLien);
				}
			}
		}

		/*
		 * Suppression des MLDEntite ne contenant qu'un identifiant
		 */
		/*
		 * Vector elementsToDel = new Vector(); for (Enumeration e =
		 * mld.elementsZElements(); e.hasMoreElements();) { ent = (MLDEntite)
		 * e.nextElement(); if (ent.sizeInformation() == 1 &&
		 * ent.sizeIdentifiant() == 1) {
		 * 
		 * for (Enumeration e2 = mld.elementsZElements(); e .hasMoreElements();) {
		 * ent2 = (MLDEntite) e.nextElement();
		 * ent2.removeForeignKey(ent.getCodeInformation(0)); }
		 * elementsToDel.add(ent); } } for (Enumeration e =
		 * elementsToDel.elements(); e.hasMoreElements();) {
		 * mld.removeElement((ZElement) e.nextElement()); }
		 */
		return true;
	}
}