/*
 * 02/26/2002 - 12:12:38
 * 
 * MCDObjet - Copyright (C) 2002 Dreux Loic dreuxl@free.fr
 * 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package org.analyse.merise.mcd.composant;

import java.awt.Font;
import java.awt.FontMetrics;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.analyse.core.gui.zgraph.ZElement;
import org.analyse.merise.gui.table.ArgObserverTable;
import org.analyse.merise.gui.table.DictionnaireTable;

public abstract class MCDObjet extends ZElement implements Observer
{
    /** Vecteur contenant les informations de l'objet */
    protected Vector informations = new Vector(30, 10);

    protected Vector links = new Vector(30, 10);

    /**
     * Numro qui permet de donner un nom temporaire au MCDEntit et
     * MCDAssociation
     */
    private static int index = 0;

    /** Nom de l'objet */
    protected String name;

    /** Sauvegarde du nom */
    private String nameSave;

    private int sizeSave;

    /** Font de l'association */
    protected Font font;

    /** Permet de calculer la taille */
    protected FontMetrics fm;

    /** Table dicos des infos */
    protected DictionnaireTable data;

    /** MCDComponent */
    protected MCDComponent mcd;

    public MCDObjet(MCDComponent mcd, String name, int x, int y, int height,
            int width)
    {
        super(mcd, x, y, width, height);
        this.name = name;
        this.nameSave = name;
        this.mcd = mcd;
        this.data = mcd.getData();
        index++;
        sizeSave = 0;
    }

    /**
     * Calcule la taille de l'objet.
     */
    public abstract void updateSize();

    /**
     * Retourne le MCD contenant le <code>MCDObjet</code>.
     */
    public MCDComponent getMCD()
    {
        return mcd;
    }

    /**
     * Vrifie si le nom  changer
     */
    public boolean isChanged()
    {
        if (name != nameSave || sizeInformation() != sizeSave) {
            nameSave = name;
            sizeSave = sizeInformation();
            return true;
        }
        return false;
    }

    /**
     * Ajoute un lien pass en paramtre.
     */
    void addLink(MCDLien link)
    {
        links.add(link);
    }

    /**
     * Supprime le lien pass en paramtre.
     */
    void delLink(MCDLien link)
    {
        links.remove(link);
    }

    /**
     * Retourne une <code>Enumeration</code> des liens.
     */
    public Enumeration links()
    {
        return links.elements();
    }

    /**
     * retourne le nombre de lien de l'objet
     */
    public int sizeLink()
    {
        return links.size();
    }

    /**
     * Retourne le numro de l'objet <br>
     * Utilis pour numrot les objets.
     */
    public static int getIndex()
    {
        return index;
    }

    /**
     * Ajouter plusieurs informations en indiquant leur code provenant du
     * dictionnaire des informations.
     * 
     * @param code
     *            Vecteur de string contenant les diffrents codes.
     */
    public void addInformations(String[] code)
    {
        for (int i = 0; i < code.length; i++)
            addInformation(code[i]);
        updateSize();
        notifyZElement();
    }

    /**
     * Ajouter une information en indiquant son code provenant du dictionnaire
     * des informations.
     * 
     * @param code
     *            code de l'information.
     */
    public void addInformation(String code)
    {
        informations.add(code);
        notifyZElement();
    }

    /**
     * Change de place les deux lments du <I>Vector informations </I>.
     * 
     * @param info1
     *            premire information  dplacer
     * @param info2
     *            deuxime information  dplacer
     */
    public void moveInformations(int info1, int info2)
    {
        Object temp;
        if (info1 < informations.size() && info2 < informations.size()) {
            temp = informations.elementAt(info1);
            informations.setElementAt(informations.elementAt(info2), info1);
            informations.setElementAt(temp, info2);
        }
        notifyZElement();
    }

    /**
     * Libre les informations du <code>MCDObjet</code>.
     */
    public void clearInformations()
    {
        for (Enumeration e = informations.elements(); e.hasMoreElements();) {
            data.setUse((String) e.nextElement(), false);
        }
        informations.clear();
        notifyZElement();
    }

    /**
     * Supprime tous les informations dont leurs codes sont passs en
     * paramtres.
     * 
     * @param code
     *            codes des informations  supprimer.
     */
    public void deleteInformations(String[] code)
    {
        for (int i = 0; i < code.length; i++)
            deleteInformation(code[i]);
        updateSize();
        notifyZElement();
    }

    /**
     * Supprime l'information dont son code est pass en paramtre.
     * 
     * @param code
     *            codes des informations  supprimer.
     */
    public void deleteInformation(String code)
    {
        data.setUse(code, false);
        if (informations.removeElement(code))
            notifyZElement();
    }

    /**
     * retourne le nombre d'informations dans l'objet.
     */
    public int sizeInformation()
    {
        return informations.size();
    }

    /**
     * retourne le code d'une informations.
     * 
     * @param i
     *            index de l'information  retourner
     */
    public String getCodeInformation(int i)
    {
        return (String) (informations.elementAt(i));
    }

    public abstract Vector getInformations();

    /**
     * Renvoie toutes les infos.
     */
    public Enumeration elementsInformations()
    {
        return informations.elements();
    }

    /**
     * Retourne le nom du <code>MCDObjet</code>
     */
    public String getName()
    {
        return name;
    }

    /**
     * Modifie le nom du <code>MCDObjet</code>
     */
    public void setName(String name)
    {
        this.name = name;
        notifyZElement();
    }

    /**
     * Initialise les numros des MCDObjet  1 lorsqu'un nouveau fichier est
     * cr
     */
    public static void clearIndex()
    {
        index = 0;
    }

    public String toString()
    {
        return "MCDObjet : " + info();
    }

    public String info()
    {
        return name + ", " + super.info();
    }

    public void update(Observable o, Object argObject)
    {
        if (argObject == null)
            return;

        ArgObserverTable arg = (ArgObserverTable) argObject;

        if (arg.getType() == ArgObserverTable.RENAME) {
            for (int i = 0; i < informations.size(); i++)
                if (((String) informations.elementAt(i)).equals(arg
                        .getOldName()))
                    informations.setElementAt(arg.getNewName(), i);
        } else if (arg.getType() == ArgObserverTable.DELETE) {
            deleteInformation(arg.getName());
        }
    }
}